@@ -23,6 +23,7 @@
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <string.h>
+#include <math.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
@@ -65,61 +66,58 @@ gd_store_set (GtkListStore *store,
 }
 
 /**
- * gd_create_sources_store:
+ * gd_create_item_store:
  * 
  * Returns: (transfer full):
  */
 GtkListStore *
-gd_create_sources_store (void)
+gd_create_item_store (void)
 {
   return gtk_list_store_new (3,
                              G_TYPE_STRING, // ID
                              G_TYPE_STRING, // NAME
-                             G_TYPE_BOOLEAN); // HEADING
+                             G_TYPE_STRING); // HEADING_TEXT
 }
 
 void
-gd_sources_store_set (GtkListStore *store,
-                      GtkTreeIter *iter,
-                      const gchar *id,
-                      const gchar *name,
-                      gboolean heading)
+gd_item_store_set (GtkListStore *store,
+                   GtkTreeIter *iter,
+                   const gchar *id,
+                   const gchar *name,
+                   const gchar *heading_text)
 {
   gtk_list_store_set (store, iter,
                       0, id,
                       1, name,
-                      2, heading,
+                      2, heading_text,
                       -1);
 }
 
 /**
- * gd_create_sidebar_store:
- *
+ * gd_create_organize_store:
+ * 
  * Returns: (transfer full):
  */
 GtkListStore *
-gd_create_sidebar_store (void)
+gd_create_organize_store (void)
 {
-  return gtk_list_store_new (4,
+  return gtk_list_store_new (3,
                              G_TYPE_STRING, // ID
                              G_TYPE_STRING, // NAME
-                             G_TYPE_STRING, // ICON
-                             G_TYPE_BOOLEAN); // HEADING
+                             G_TYPE_INT); // STATE
 }
 
 void
-gd_sidebar_store_set (GtkListStore *store,
-                      GtkTreeIter *iter,
-                      const gchar *id,
-                      const gchar *name,
-                      const gchar *icon_name,
-                      gboolean heading)
+gd_organize_store_set (GtkListStore *store,
+                       GtkTreeIter *iter,
+                       const gchar *id,
+                       const gchar *name,
+                       gint state)
 {
   gtk_list_store_set (store, iter,
                       0, id,
                       1, name,
-                      2, icon_name,
-                      3, heading,
+                      2, state,
                       -1);
 }
 
@@ -259,131 +257,89 @@ gd_gtk_tree_view_set_activate_on_single_click (GtkTreeView *tree_view,
 	}
 }
 
-/* utility to stretch a frame to the desired size */
-
-static void
-draw_frame_row (GdkPixbuf *frame_image, int target_width, int source_width, int source_v_position, int dest_v_position, GdkPixbuf *result_pixbuf, int left_offset, int height)
-{
-	int remaining_width, h_offset, slab_width;
-	
-	remaining_width = target_width;
-	h_offset = 0;
-	while (remaining_width > 0) {	
-		slab_width = remaining_width > source_width ? source_width : remaining_width;
-		gdk_pixbuf_copy_area (frame_image, left_offset, source_v_position, slab_width, height, result_pixbuf, left_offset + h_offset, dest_v_position);
-		remaining_width -= slab_width;
-		h_offset += slab_width; 
-	}
-}
-
-/* utility to draw the middle section of the frame in a loop */
-static void
-draw_frame_column (GdkPixbuf *frame_image, int target_height, int source_height, int source_h_position, int dest_h_position, GdkPixbuf *result_pixbuf, int top_offset, int width)
-{
-	int remaining_height, v_offset, slab_height;
-	
-	remaining_height = target_height;
-	v_offset = 0;
-	while (remaining_height > 0) {	
-		slab_height = remaining_height > source_height ? source_height : remaining_height;
-		gdk_pixbuf_copy_area (frame_image, source_h_position, top_offset, width, slab_height, result_pixbuf, dest_h_position, top_offset + v_offset);
-		remaining_height -= slab_height;
-		v_offset += slab_height; 
-	}
-}
-
-static GdkPixbuf *
-gd_stretch_frame_image (GdkPixbuf *frame_image, int left_offset, int top_offset, int right_offset, int bottom_offset,
-			 int dest_width, int dest_height, gboolean fill_flag)
-{
-	GdkPixbuf *result_pixbuf;
-	guchar *pixels_ptr;
-	int frame_width, frame_height;
-	int y, row_stride;
-	int target_width, target_frame_width;
-	int target_height, target_frame_height;
-	
-	frame_width  = gdk_pixbuf_get_width  (frame_image);
-	frame_height = gdk_pixbuf_get_height (frame_image );
-	
-	if (fill_flag) {
-		result_pixbuf = gdk_pixbuf_scale_simple (frame_image, dest_width, dest_height, GDK_INTERP_NEAREST);
-	} else {
-		result_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, dest_width, dest_height);
-	}
-	row_stride = gdk_pixbuf_get_rowstride (result_pixbuf);
-	pixels_ptr = gdk_pixbuf_get_pixels (result_pixbuf);
-	
-	/* clear the new pixbuf */
-	if (!fill_flag) {
-		for (y = 0; y < dest_height; y++) {
-			memset (pixels_ptr, 255, row_stride);
-			pixels_ptr += row_stride; 
-		}
-	}
-	
-	target_width  = dest_width - left_offset - right_offset;
-	target_frame_width = frame_width - left_offset - right_offset;
-	
-	target_height  = dest_height - top_offset - bottom_offset;
-	target_frame_height = frame_height - top_offset - bottom_offset;
-	
-	/* draw the left top corner  and top row */
-	gdk_pixbuf_copy_area (frame_image, 0, 0, left_offset, top_offset, result_pixbuf, 0,  0);
-	draw_frame_row (frame_image, target_width, target_frame_width, 0, 0, result_pixbuf, left_offset, top_offset);
-	
-	/* draw the right top corner and left column */
-	gdk_pixbuf_copy_area (frame_image, frame_width - right_offset, 0, right_offset, top_offset, result_pixbuf, dest_width - right_offset,  0);
-	draw_frame_column (frame_image, target_height, target_frame_height, 0, 0, result_pixbuf, top_offset, left_offset);
-
-	/* draw the bottom right corner and bottom row */
-	gdk_pixbuf_copy_area (frame_image, frame_width - right_offset, frame_height - bottom_offset, right_offset, bottom_offset, result_pixbuf, dest_width - right_offset,  dest_height - bottom_offset);
-	draw_frame_row (frame_image, target_width, target_frame_width, frame_height - bottom_offset, dest_height - bottom_offset, result_pixbuf, left_offset, bottom_offset);
-		
-	/* draw the bottom left corner and the right column */
-	gdk_pixbuf_copy_area (frame_image, 0, frame_height - bottom_offset, left_offset, bottom_offset, result_pixbuf, 0,  dest_height - bottom_offset);
-	draw_frame_column (frame_image, target_height, target_frame_height, frame_width - right_offset, dest_width - right_offset, result_pixbuf, top_offset, right_offset);
-	
-	return result_pixbuf;
-}
-
 /**
  * gd_embed_image_in_frame: 
  * @source_image:
- * @frame_image:
- * @left_offset:
- * @top_offset:
- * @right_offset:
- * @bottom_offset:
+ * @frame_image_path:
+ * @slice_width:
+ * @border_width:
  *
  * Returns: (transfer full):
  */
-/* draw an arbitrary frame around an image, with the result passed back in a newly allocated pixbuf */
 GdkPixbuf *
 gd_embed_image_in_frame (GdkPixbuf *source_image,
-                         GdkPixbuf *frame_image,
-                         int left_offset, 
-                         int top_offset, 
-                         int right_offset, 
-                         int bottom_offset)
+                         const gchar *frame_image_path,
+                         GtkBorder *slice_width,
+                         GtkBorder *border_width)
 {
-	GdkPixbuf *result_pixbuf;
-	int source_width, source_height;
-	int dest_width, dest_height;
-	
-	source_width  = gdk_pixbuf_get_width  (source_image);
-	source_height = gdk_pixbuf_get_height (source_image);
+  cairo_surface_t *surface;
+  cairo_t *cr;
+  int source_width, source_height;
+  int dest_width, dest_height;
+  gchar *css_str;
+  GtkCssProvider *provider;
+  GtkStyleContext *context;
+  GError *error = NULL;
+  GdkPixbuf *retval;
+  GtkWidgetPath *path;
+ 
+  source_width = gdk_pixbuf_get_width (source_image);
+  source_height = gdk_pixbuf_get_height (source_image);
 
-	dest_width  = source_width  + left_offset + right_offset;
-	dest_height = source_height + top_offset  + bottom_offset;
-	
-	result_pixbuf = gd_stretch_frame_image (frame_image, left_offset, top_offset, right_offset, bottom_offset, 
-                                                dest_width, dest_height, FALSE);
-		
-	/* Finally, copy the source image into the framed area */
-	gdk_pixbuf_copy_area (source_image, 0, 0, source_width, source_height, result_pixbuf, left_offset,  top_offset);
+  dest_width = source_width +  border_width->left + border_width->right;
+  dest_height = source_height + border_width->top + border_width->bottom;
 
-	return result_pixbuf;
+  css_str = g_strdup_printf (".embedded-image { border-image: url(\"%s\") %d %d %d %d / %d %d %d %d }",
+                             frame_image_path, 
+                             slice_width->top, slice_width->right, slice_width->bottom, slice_width->left,
+                             border_width->top, border_width->right, border_width->bottom, border_width->left);
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_data (provider, css_str, -1, &error);
+
+  if (error != NULL) 
+    {
+      g_warning ("Unable to create the thumbnail frame image: %s", error->message);
+      g_error_free (error);
+      g_free (css_str);
+
+      return g_object_ref (source_image);
+    }
+
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, dest_width, dest_height);
+  cr = cairo_create (surface);
+
+  context = gtk_style_context_new ();
+  path = gtk_widget_path_new ();
+  gtk_widget_path_append_type (path, GTK_TYPE_ICON_VIEW);
+
+  gtk_style_context_set_path (context, path);
+  gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), 600);
+
+  gtk_style_context_save (context);
+  gtk_style_context_add_class (context, "embedded-image");
+
+  gtk_render_frame (context, cr,
+                    0, 0,
+                    dest_width, dest_height);
+
+  gtk_style_context_restore (context);
+
+  gtk_render_icon (context, cr,
+                   source_image,
+                   border_width->left, border_width->top);
+
+  retval = gdk_pixbuf_get_from_surface (surface,
+                                        0, 0, dest_width, dest_height);
+
+  cairo_surface_destroy (surface);
+  cairo_destroy (cr);
+
+  gtk_widget_path_unref (path);
+  g_object_unref (provider);
+  g_object_unref (context);
+  g_free (css_str);
+
+  return retval;
 }
 
 static char *
@@ -451,6 +407,115 @@ gd_time_val_from_iso8601 (const gchar *string,
   return g_time_val_from_iso8601 (string, timeval);
 }
 
+/**
+ * gd_iso8601_from_timestamp:
+ * @timestamp:
+ *
+ * Returns: (transfer full):
+ */
+gchar *
+gd_iso8601_from_timestamp (gint64 timestamp)
+{
+  GTimeVal tv;
+
+  tv.tv_sec = timestamp;
+  tv.tv_usec = 0;
+  return g_time_val_to_iso8601 (&tv);
+}
+
+/**
+ * gd_create_collection_icon:
+ * @base_size:
+ * @pixbufs: (element-type GdkPixbuf):
+ *
+ * Returns: (transfer full):
+ */
+GIcon *
+gd_create_collection_icon (gint base_size,
+                           GList *pixbufs)
+{
+  cairo_surface_t *surface;
+  GIcon *retval;
+  cairo_t *cr;
+  GtkStyleContext *context;
+  GtkWidgetPath *path;
+  gint padding, tile_size, scale_size;
+  gint pix_width, pix_height;
+  gint idx, cur_x, cur_y;
+  GList *l;
+  GdkPixbuf *pix;
+
+  /* TODO: do not hardcode 4, but scale to another layout if more
+   * pixbufs are provided.
+   */
+
+  padding = MAX (floor (base_size / 10), 4);
+  tile_size = (base_size - (3 * padding)) / 2;
+
+  context = gtk_style_context_new ();
+  gtk_style_context_add_class (context, "documents-collection-icon");
+
+  path = gtk_widget_path_new ();
+  gtk_widget_path_append_type (path, GTK_TYPE_ICON_VIEW);
+  gtk_style_context_set_path (context, path);
+  gtk_widget_path_unref (path);
+
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, base_size, base_size);
+  cr = cairo_create (surface);
+
+  gtk_render_background (context, cr,
+                         0, 0, base_size, base_size);
+
+  l = pixbufs;
+  idx = 0;
+  cur_x = padding;
+  cur_y = padding;
+
+  while (l != NULL && idx < 4)
+    {
+      pix = l->data;
+      pix_width = gdk_pixbuf_get_width (pix);
+      pix_height = gdk_pixbuf_get_height (pix);
+
+      scale_size = MIN (pix_width, pix_height);
+
+      cairo_save (cr);
+
+      cairo_translate (cr, cur_x, cur_y);
+
+      cairo_rectangle (cr, 0, 0,
+                       tile_size, tile_size);
+      cairo_clip (cr);
+
+      cairo_scale (cr, (gdouble) tile_size / (gdouble) scale_size, (gdouble) tile_size / (gdouble) scale_size);
+      gdk_cairo_set_source_pixbuf (cr, pix, 0, 0);
+
+      cairo_paint (cr);
+      cairo_restore (cr);
+
+      if ((idx % 2) == 0)
+        {
+          cur_x += tile_size + padding;
+        }
+      else
+        {
+          cur_x = padding;
+          cur_y += tile_size + padding;
+        }
+
+      idx++;
+      l = l->next;
+    }
+
+  retval = G_ICON (gdk_pixbuf_get_from_surface (surface, 0, 0, base_size, base_size));
+
+  cairo_surface_destroy (surface);
+  cairo_destroy (cr);
+  g_object_unref (context);
+
+  return retval;
+}
+
 #define _BG_MIN_SIZE 20
 #define _EMBLEM_MIN_SIZE 8
 
@@ -522,3 +587,68 @@ gd_create_symbolic_icon (const gchar *name,
 
   return retval;
 }
+
+/* taken from gtk/gtktreeview.c */
+static void
+send_focus_change (GtkWidget *widget,
+                   GdkDevice *device,
+		   gboolean   in)
+{
+  GdkDeviceManager *device_manager;
+  GList *devices, *d;
+
+  device_manager = gdk_display_get_device_manager (gtk_widget_get_display (widget));
+  devices = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
+  devices = g_list_concat (devices, gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_SLAVE));
+  devices = g_list_concat (devices, gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_FLOATING));
+
+  for (d = devices; d; d = d->next)
+    {
+      GdkDevice *dev = d->data;
+      GdkEvent *fevent;
+      GdkWindow *window;
+
+      if (gdk_device_get_source (dev) != GDK_SOURCE_KEYBOARD)
+        continue;
+
+      window = gtk_widget_get_window (widget);
+
+      /* Skip non-master keyboards that haven't
+       * selected for events from this window
+       */
+      if (gdk_device_get_device_type (dev) != GDK_DEVICE_TYPE_MASTER &&
+          !gdk_window_get_device_events (window, dev))
+        continue;
+
+      fevent = gdk_event_new (GDK_FOCUS_CHANGE);
+
+      fevent->focus_change.type = GDK_FOCUS_CHANGE;
+      fevent->focus_change.window = g_object_ref (window);
+      fevent->focus_change.in = in;
+      gdk_event_set_device (fevent, device);
+
+      gtk_widget_send_focus_change (widget, fevent);
+
+      gdk_event_free (fevent);
+    }
+
+  g_list_free (devices);
+}
+
+void
+gd_entry_focus_hack (GtkWidget *entry,
+                     GdkDevice *device)
+{
+  GtkEntryClass *entry_class;
+  GtkWidgetClass *entry_parent_class;
+
+  /* Grab focus will select all the text.  We don't want that to happen, so we
+   * call the parent instance and bypass the selection change.  This is probably
+   * really non-kosher. */
+  entry_class = g_type_class_peek (GTK_TYPE_ENTRY);
+  entry_parent_class = g_type_class_peek_parent (entry_class);
+  (entry_parent_class->grab_focus) (entry);
+
+  /* send focus-in event */
+  send_focus_change (entry, device, TRUE);
+}
