@@ -30,22 +30,22 @@ void gd_store_set (GtkListStore *store,
                    const gchar *urn,
                    const gchar *title,
                    const gchar *author,
-                   GdkPixbuf *icon);
+                   GdkPixbuf *icon,
+                   glong mtime);
 
-GtkListStore* gd_create_sources_store (void);
-void gd_sources_store_set (GtkListStore *store,
-                           GtkTreeIter *iter,
-                           const gchar *id,
-                           const gchar *name,
-                           gboolean heading);
+GtkListStore* gd_create_item_store (void);
+void gd_item_store_set (GtkListStore *store,
+                        GtkTreeIter *iter,
+                        const gchar *id,
+                        const gchar *name,
+                        const gchar *heading_text);
 
-GtkListStore * gd_create_sidebar_store (void);
-void gd_sidebar_store_set (GtkListStore *store,
-                           GtkTreeIter *iter,
-                           const gchar *id,
-                           const gchar *name,
-                           const gchar *icon_name,
-                           gboolean heading);
+GtkListStore* gd_create_organize_store (void);
+void gd_organize_store_set (GtkListStore *store,
+                            GtkTreeIter *iter,
+                            const gchar *id,
+                            const gchar *name,
+                            gint state);
 
 void gd_queue_thumbnail_job_for_file_async (GFile *file,
                                             GAsyncReadyCallback callback,
@@ -56,14 +56,23 @@ gboolean gd_queue_thumbnail_job_for_file_finish (GAsyncResult *res);
 void gd_gtk_tree_view_set_activate_on_single_click (GtkTreeView *tree_view,
                                                     gboolean should_activate);
 
-GdkPixbuf * gd_embed_image_in_frame (GdkPixbuf *source_image,
-                                     GdkPixbuf *frame_image,
-                                     int left_offset,
-                                     int top_offset,
-                                     int right_offset,
-                                     int bottom_offset);
+GdkPixbuf *gd_embed_image_in_frame (GdkPixbuf *source_image,
+                                    const gchar *frame_image_path,
+                                    GtkBorder *slice_width,
+                                    GtkBorder *border_width);
 
 char *gd_filename_strip_extension (const char * filename_with_extension);
 
+gboolean gd_time_val_from_iso8601 (const gchar *string,
+                                   GTimeVal *timeval);
+gchar *gd_iso8601_from_timestamp (gint64 timestamp);
+
+GIcon *gd_create_collection_icon (gint base_size,
+                                  GList *pixbufs);
+GIcon *gd_create_symbolic_icon (const gchar *name,
+                                gint base_size);
+void   gd_entry_focus_hack (GtkWidget *entry,
+                            GdkDevice *device);
+
 #endif /* __GD_UTILS_H__ */
                                   
